#ifndef HOPALONG_H
#define HOPALONG_H

// Interface to Hopalong screensaver class


#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include <math.h>
#include <ScreenSaver.h>

#include "CreditsView.h"


// DEFINED CONSTANTS
// Rate at which we move around the color wheel
#define STEPRATE (0.001)
#define PREVIEW_STEPRATE (0.1)

// Restrict the saver to one time round the color wheel 
#define MAX_ITERS (360.0/STEPRATE)
#define PREVIEW_MAX_ITERS (360.0/PREVIEW_STEPRATE)

// Time to sleep between iterations
#define TICKSIZE 5000



// Class is derived from the BScreenSaver base class
class HopalongSaver : public BScreenSaver
{
  private:
    // Keeps track of iterations
    unsigned int iter_count;
  
  
    // Hopalong fractal parameters
    typedef struct
    {
      float a;
      float b;
      float c;

      float x;
      float y;
    } hoppy;

    // Our Hopalong fractal
    hoppy hopper;
    
    
    // HSV color value, with alpha component
    typedef struct
    {
      float h; // 0 to 360 (it's in degrees)
      float s;
      float v;
      float alpha;
    } hsv_color;

    // Converts HSV color values to RGB
    rgb_color hsv_to_rgb(hsv_color &);

    // HSV color (so we can select color via shifting hues...)
    hsv_color hsv_draw_color;
    
    // Cache the color and only update it if we move more than a tenth of a
    //  degree around the color wheel
    float last_hue;
    rgb_color last_rgb_color;
  
    // Values used to scale and position the fractal on the screen
    float offset_width, offset_height, scale_width, scale_height;

    // Keeps track of whether we're in preview mode or not
    bool is_preview;


  public:
    // Constructor
    HopalongSaver( BMessage *archive, image_id image );    
    
    // Override virtual function implementations from base class
    void StartConfig( BView *view ); 
    status_t StartSaver( BView *view, bool preview ); 
    void Draw( BView *view, int32 frame );        
};



#endif



